"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.formatVariantSelector = formatVariantSelector;
exports.finalizeSelector = finalizeSelector;
exports.selectorFunctions = void 0;
var _postcssSelectorParser = _interopRequireDefault(require("postcss-selector-parser"));
var _unesc = _interopRequireDefault(require("postcss-selector-parser/dist/util/unesc"));
var _escapeClassName = _interopRequireDefault(require("../util/escapeClassName"));
var _prefixSelector = _interopRequireDefault(require("../util/prefixSelector"));
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
let MERGE = ':merge';
let PARENT = '&';
let selectorFunctions = new Set([
    MERGE
]);
exports.selectorFunctions = selectorFunctions;
function formatVariantSelector(current, ...others) {
    for (let other of others){
        let incomingValue = resolveFunctionArgument(other, MERGE);
        if (incomingValue !== null) {
            let existingValue = resolveFunctionArgument(current, MERGE, incomingValue);
            if (existingValue !== null) {
                let existingTarget = `${MERGE}(${incomingValue})`;
                let splitIdx = other.indexOf(existingTarget);
                let addition = other.slice(splitIdx + existingTarget.length).split(' ')[0];
                current = current.replace(existingTarget, existingTarget + addition);
                continue;
            }
        }
        current = other.replace(PARENT, current);
    }
    return current;
}
function finalizeSelector(format, { selector , candidate , context  }) {
    var ref, ref1;
    var ref2;
    let base = candidate.split((ref2 = context === null || context === void 0 ? void 0 : (ref = context.tailwindConfig) === null || ref === void 0 ? void 0 : ref.separator) !== null && ref2 !== void 0 ? ref2 : ':').pop();
    if (context === null || context === void 0 ? void 0 : (ref1 = context.tailwindConfig) === null || ref1 === void 0 ? void 0 : ref1.prefix) {
        format = (0, _prefixSelector).default(context.tailwindConfig.prefix, format);
    }
    format = format.replace(PARENT, `.${(0, _escapeClassName).default(candidate)}`);
    // Normalize escaped classes, e.g.:
    //
    // The idea would be to replace the escaped `base` in the selector with the
    // `format`. However, in css you can escape the same selector in a few
    // different ways. This would result in different strings and therefore we
    // can't replace it properly.
    //
    //               base: bg-[rgb(255,0,0)]
    //   base in selector: bg-\\[rgb\\(255\\,0\\,0\\)\\]
    //       escaped base: bg-\\[rgb\\(255\\2c 0\\2c 0\\)\\]
    //
    selector = (0, _postcssSelectorParser).default((selectors)=>{
        return selectors.walkClasses((node)=>{
            if (node.raws && node.value.includes(base)) {
                node.raws.value = (0, _escapeClassName).default((0, _unesc).default(node.raws.value));
            }
            return node;
        });
    }).processSync(selector);
    // We can safely replace the escaped base now, since the `base` section is
    // now in a normalized escaped value.
    selector = selector.replace(`.${(0, _escapeClassName).default(base)}`, format);
    // Remove unnecessary pseudo selectors that we used as placeholders
    return (0, _postcssSelectorParser).default((selectors)=>{
        return selectors.map((selector)=>{
            selector.walkPseudos((p)=>{
                if (selectorFunctions.has(p.value)) {
                    p.replaceWith(p.nodes);
                }
                return p;
            });
            return selector;
        });
    }).processSync(selector);
}
function resolveFunctionArgument(haystack, needle, arg) {
    let startIdx = haystack.indexOf(arg ? `${needle}(${arg})` : needle);
    if (startIdx === -1) return null;
    // Start inside the `(`
    startIdx += needle.length + 1;
    let target = '';
    let count = 0;
    for (let char of haystack.slice(startIdx)){
        if (char !== '(' && char !== ')') {
            target += char;
        } else if (char === '(') {
            target += char;
            count++;
        } else if (char === ')') {
            if (--count < 0) break; // unbalanced
            target += char;
        }
    }
    return target;
}
